/*

Setting1 -> 	POV(Char,Loc,PC-Verb) +      %Char's feeling about loc
		Location Features(PC-Verb) + %PC-Verb here determines charge +/-
		POV(Char,Loc,PC-Verb,NegFeatureLack.Counter-Setting) +
		Forboding-Contrast(Loc,NegFeature,Counter-Setting).
		Example(NegFeature,Example) + 
		Purpose(Example,Setting) + 
		POV(Antagonist,Example).

Input into the setting1 generator.
	Char = Protagonist (Betrayed),
	Loc  = Principal Location of Setting
	PC-Verb = P-Consciousness verb used to describe Char's feeling about 
			setting. These are classified as positive or negative.
			Feature selection in loc description will depend on charge.
	Negative-Feature = 




Describing the Location as part of the Establishing the Setting

   We can describe a location by listing and modifying
   its attributes. We can do this objectively with linking
   words like L 'has' or 'contains' or 'is full of'. We can
   do it subjectively from someone,e.g., the main character's POV. Here
   linking words would reflect p-conciousness like "loves" L's ... or
   "hates" L's .... We can choose this word based on the subjects psychological
   characteristics -- how he "feels" about a location 
	(likes, loves, hates, has enjoyed). Of course if a character hates a 
   location then the reader expects some explanation for why the character is there.

   Each object in the KB may have classes of linguistic modifiers associated with it.
   For example, clock-tower might have size modifers big, tall etc. shape and color
   modifiers that apply etc.
 
 

*/

%======================================================
% Write Sentence
%	Given the name of a sentence frame builds
%	a instance of that sentence returned as a list.
%======================================================

%---------------------------------------------
% Write a PARALLEL_SETTING sentence
%---------------------------------------------
write_sentence(parallel_setting,Sentence) :-
	lookup(setting,Sentence_Frame,Location),
	lookup(point_of_view,Sentence_Frame,POV),
	lookup(pc_verb,Sentence_Frame,Verb),
	lookup(tense,Sentence_Frame,Tense),
	lookup(code,Sentence_Frame,P),
	P(Location,POV,Verb,Tense,Sentence).
		
		

%======================================================
% Describe Setting.
%
%   Uses parallel structue to shape a 
%   sentence describing a setting location.
%   Randomly selects iconic features of
%   the location, modifying them and then
%   building a sentence connecting the list
%   to the location through a randonly selected
%   linking verb.
%  
%
%  Variations: Objective, POV(Character)
%
%=========================================

describe_setting(Loc,objective,Loc,S) :-
	nl,
	meta_splice(['The', Loc],NP),
	choose_verb(feature_link,Verb),
	build_modified_feature_list(Loc,MFL1),
	insert_and(MFL1,MFL2),
	meta_splice(MFL2,MFL),
	meta_splice([NP,Verb,MFL],S),
	writeseqnl(S).

%----------------------------------------------------------------------------------
% Builds a sentence about a setting highlighting
% a negative or threating feature of the setting. Used
% in contrast to previous mentioned positive features
% to warn the reader.
%
%   "Of course, the S has its *downside*. It has its
%    N1, N2 and N3. S's N2 can be as *merciless* as they
%    are in *worse place ever for N3s*."
%
%    NXs are features. Features should have iconic objects
%    -- things where that feature is highly suggestive. For
%    example, tests are highly suggestive in the context of
%    schools as opposed to playgrounds. This object of place
%    that features the negative aspect of the setting is called
%    the counter-setting of the setting with respect to the
%    feature.
%    For example, S = university, Negative-Feature(NF) = trials
%       counter-setting(S,NF) = marketplace.
%----------------------------------------------------------------------------------
forboding_contrast().


%----------------------------------------------------------------------------------
% Paragaph Grammar for Describing Setting From Character's Point of View
%
% Describe Setting based on characters Point of View.
%	Sentence 1: Character Postitive PC Verb the Setting
%	Sentence 2: Character Positive PC Verb Positive Features of Setting.
%	Sentence 3: Additional  Positive PC Verb for lack of negative feature
%
%
% Must Choose P-Conciousness verb to establish
% characters conscious awareness of setting.
%
% Currently only present and past tense supported
%
%  TODO: Must decide to use positive or negative
%        literay modifiers based on "charge" of
%        PCVerb.
%
%
% Example: "	Selmer loved the university. 
%		He loved its ivy-covered clocktowers, its ancient and sturdy brick
%		and its sun-splased verdant greens and eager youth. "
%
%----------------------------------------------------------------------------------
dspov(Location,Character,PCVerb,Tense,Sentences) :-
	tense(PCVerb,Tense,Verb),

	%Sentence 1: 	Prototype -- Dave Striver loved the University
	%		Template -- <Character> <PCVerb> <Article> <Setting>	
	%	
	meta_splice([Character,Verb,`the`, Location],S1),

	pp_sentence(S1),

	%Sentence 2:	Prototype -- He loved its ivy-covered clocktowers,
	%				its ancient and sturdy brick, and its sub-splashed verdant
	%				greens and eagar youth.
	%
	%		Template  -- <Character Pronoun> <PCVerb> <Modified Feature NP for Setting>
	% 
	%		
	get_pronoun(Character,sub,CPro),		%Subjective Pronoun
	get_pronoun(Location,pos,LPro),			%Possesive Pronoun

	build_possesive_modified_feature_list(Location,LPro,MFL),
	meta_splice([CPro,Verb | MFL],S2),

	pp_sentence(S2),
	
	append(S1,S2,SentenceBlock1),

	setting_contrast(Location,Character,PCVerb,Tense,SentenceBlock2),

	append(SentenceBlock1,SentenceBlock2,Sentences).




%==============================================================================================
% Setting Contrast Sentence Block

	%Prototypes -- S1: He also loved the fact that the univeristy is free of the
	%		stark unforgiving trials of the business world -- 
	%		only this isn't a fact.
	%
	%
	%		S2: Academia has its own tests, just as merciless as any in the marketplace. 
	%			
	%
	%Template -- <Character Pronoun> <Connective> <PCVerb> <relative pronoun phrase>
	%		<Setting> <Subtraction Phrase> <Contrasting Setting NP>
	%		<Contradiction Assertion>
	%
	%		<Setting Synonym> <Possess Phrase with Setting Negative Feature>,
	%		just as <TERRIBLE> as any in the <counter setting synonym>
	%
%==============================================================================================
setting_contrast(Setting,Character,PCVerb, Tense, Sentences) :-	
	%Must return counter setting as well
	%
	positive_pov_of_negative_setting_fact(Location,Character,PCVerb,Tense,CounterSetting,NegativeFeature,P1),
	contradiction_assertion(AS),
	meta_splice([P1,`--`, AS], Sentence1),

	pp_sentence(Sentence1),
	
	synonym(Setting,AlternateSetting),
	synonym(NegativeFeature, AlternateNF),

	additional_ownership(AlternateSetting,AlternateNF,present,Sentence2Fragment1),

	modifiers(negative, AlternateNF, Modifiers),

	randomly_select_at_most(1,Modifiers,[ANFModifier]),

	synonym(CounterSetting,SCS),
	meta_splice([`just as`, ANFModifier,`as any in`,`the`,SCS],Sentence2Fragment2),

	meta_splice([Sentence2Fragment1,`,`,Sentence2Fragment2],Sentence2),

	pp_sentence(Sentence2),
	
	append(Sentence1,Sentence2,Sentences).
	
	

%-----------------------------------------------
%
%-----------------------------------------------
contradiction_assertion(`only this isn't true`).
contradiction_assertion(`only this isn't a fact`).
contradiction_assertion(`not true`).

%==============================================================================================
%
%==============================================================================================
additional_ownership(Subject, Object, present, S) :-
	meta_splice([Subject,`has its own`, Object],S).

additional_ownership(Subject, Object, present, S) :-
	randomly_select_at_most(1,[too, also, `as well`],[MeTooWord]),
	meta_splice([Subject,`has its`, Object, MeTooWord],S).



%==============================================================================================

	% 	Prototype -- " He also loved the fact that the unviversity is free of the stark 
	%			unforgiving trials of the business world."

	% (+Location,+Character,+PCVerb,+Tense,-ContrastingObject,-NegativeFeature,-Sentence)
%==============================================================================================
positive_pov_of_negative_setting_fact(Location,Character,PCVerb,Tense,ContrastingObject,NegativeFeature,Sentence) :-
	tense(PCVerb,Tense,Verb),
	get_pronoun(Character,sub,CPro),
	subtraction_phrase(present,SubPhrase),
	
	% Should generalize notion of location and more than that should build noun phrase
	% given location to determine proper article agreement.
	%
	one(counter_setting(Location,NegativeFeature,ContrastingObject)),	
	feature_modify_prep_possesive(ContrastingObject,CNP),
	
	meta_splice([CPro, `also`, Verb, `the fact that the`,Location,SubPhrase,CNP],Sentence).



%==============================================================================================
%==============================================================================================
get_subtraction_phrase(Tense,SubPhrase) :-
	findall(P,subraction_phrase(Tense,P),Ps),
	randomly_select_at_most(1,Ps,[SubPhrase]).


%----------------------------------------
%subtraction_phrase(+Tense,?Verb_Phrase)
%----------------------------------------
subtraction_phrase(present,`is free of`).
subtraction_phrase(present, `lacks`).
subtraction_phrase(present,`does not have`).




%=================================================
% Sentence Type: Describe Persons Goal
%
%  Build a sentence that links a person to his/her
%  Goal.
%  Assumptions:
%	1) Goal is a expressed as a verb
%
% Example: "Selmer wants to learn."
%
%=================================================
describePersonsGoal(Person,Goal,Tense,[Person,LinkVerb,GoalVerb]) :-
	choose_verb(goal_link,Verb),
	tense(Verb, Tense, LinkVerb),
	tense(Goal,infinitive,GoalVerb).


%=================================================
% Sentence Type: General Simple Sentence
%
%  Build a Subject Verb Object Phrase Sentence
%  Assumptions:
%
%=================================================
simpleSentence(Agent,Action,Tense,Object_Phrase,[Agent, Verb_Word, Object_Phrase]) :-
	lookup(Tense, Action, Verb_Word).

			
%=========================================
%
%
%=========================================
build_possesive_modified_feature_list(Object,PPro,PMFL) :-
	build_modified_feature_list(Object,MFL),
	map_possesive(MFL,PPro,PMFL1),
	insert_and(PMFL1,PMFL2),
	meta_splice(PMFL2,PMFL).

%=========================================

/*
 Build Prepositional Possesive
Example. the stark, unforgiving trials of the business world...
where "the stark, unforgivg trials" is a modified feature list of "the business world".



TODO: We want 1 feature modifier but we want to parameterize the number of modifiers
on that feature

*/
%=========================================
feature_modify_prep_possesive(Object,NP) :-
	build_modified_feature_list(Object,Charge,1,MFL), 
	%writenl(MFL),
	meta_splice([`the`,MFL,`of the`,Object],NP).



%=========================================
%
% Default Call to Modify features of
% an object is "Positive" and "3" features
%
% Given Object, select at most 3 positive
% features and for each selects a 
% single modifier and builds the phrase.
%
%=========================================
build_modified_feature_list(Object,MFL) :-
	build_modified_feature_list(Object,positive,3,MFL).


%=========================================
%
% Assumes 'charge' on features is consistent
% with charge of feature modifiers.
% For example, a negative feature of
% a university is 'tests' tests can
% be modified positively 'good tests' or
% negatively 'painful tests', we assume the charge
% coming in on the feature.
%
%=========================================
build_modified_feature_list(Object,Charge,NumberOfFeatures,MFL) :-
	icons(Charge,Object,Features),
	randomly_select_at_most(NumberOfFeatures,Features,NFeatures),
	modify_list(Object,NFeatures,Charge,MFL).


% TODO: maybe have a clause where rather than a number for NumberOfFeatures
% we have (NumberOfFeatures, NumberOfModifiersPerFeature)
% then call modify_list with NumberOfModifiersPerFeature

 
%********************************************************
%
% 	Modified Feature List Operators
%
% (TODO: Clauses should be named MODIFY_FEATURE because
% 	it is predicated on the featured_object as well).
%
%********************************************************

%=========================================
% 
%=========================================
map_possesive(MFL,PPro,R) :-
	findall(V,( member(MF,MFL),V = [PPro | MF]),R).

%=========================================
% 
% Inserts an 'and' at beggining of
% last pair in list. Pairs are intended
% to represent [modifier object] pairs
% that will be listed in a sentence.
%
%=========================================
insert_and(LOL,R) :-
	reverse(LOL,RLOL),
	=(RLOL,[F | Rest]),
	append([and],F,NewLast),
	reverse([NewLast | Rest],R).


%====================================================
% MODIFY_LIST of Features
%
% Assign modifiers to each feature in a feature list
%
%====================================================
modify_list(Object, Features, Charge, ModifiedFeatures) :-
	modify1(Object, Features, Charge, [],ModifiedFeatures).

modify1(Object, [], Charge, ModifiedFeatures, ModifiedFeatures) :- !.

modify1(Object, [F | Features],Charge, ACCUM,ModifiedFeatures) :-
	modify(Object, F, Charge, MF),
	modify1(Object, Features, Charge, [MF | ACCUM], ModifiedFeatures).


%================================================================
% MODIFY Feature 
%
% We can enhance this by constraining
% the selection of modifiers based on
% the object that has the feature.
% For example, university sports might
% be modified differently than professional sports.
%
% The Word Usage Pattern to use for this is the feature_modifier,
% rather than just the modifier,
% which is predicated on the Object as well as its feature.
%
%
% If the Feature can NOT be modified based on 
% knowledge base then default modifier is 'interesting'.
%
%================================================================

%--------------------------------------------------
% Charge Must be bound to positive or negative.
%
%--------------------------------------------------

%
% Selects at most 1 modifier for Feature
%
modify(Object, Feature,Charge, [Modifier,Feature]) :-
	all_feature_modifiers(Charge, Object, Feature, PLMs),
	not(=(PLMs,[])),!,
	randomly_select_at_most(1,PLMs,[Modifier]).

%
% Selects at most specified number of modifiers for Feature
%
modify(Object,Feature,Charge,NumberOfModifiers, MFL) :-
	all_feature_modifiers(Charge, Object, Feature, PLMs),
	not(=(PLMs,[])),!,
	randomly_select_at_most(NumberOfModifiers,PLMs,Modifiers),
	append(Modifiers,Feature,MFL).

%--------------------------------------------------
% MODIFY Feature where feature was not recognized
%	and may need to be depluralized.
%
%--------------------------------------------------
modify(Object, Feature,Charge, [Modifier,Feature]) :-
	depluralize(Feature,DePFeature),
	isa_frame(DePFeature,_),
	all_feature_modifiers(Charge,Object,Feature,PLMs),!,
	randomly_select_at_most(1,PLMs,[Modifier]).

%--------------------------------------------------
% MODIFY Feature where feature is not a recognized
%	frame. Default modifier is 'interesing'.
%--------------------------------------------------
modify(Object, Feature, Charge, [interesting,Feature]).
