/*

Setting1 -> 	POV(Char,Loc,PC-Verb) +      %Char's feeling about loc
		Location Features(PC-Verb) + %PC-Verb here determines charge +/-
		POV(Char,Loc,PC-Verb,NegFeatureLack.Counter-Setting) +
		Forboding-Contrast(Loc,NegFeature,Counter-Setting).
		Example(NegFeature,Example) + 
		Purpose(Example,Setting) + 
		POV(Antagonist,Example).

Input into the setting1 generator.
	Char = Protagonist (Betrayed),
	Loc  = Principal Location of Setting
	PC-Verb = P-Consciousness verb used to describe Char's feeling about 
			setting. These are classified as positive or negative.
			Feature selection in loc description will depend on charge.
	Negative-Feature = 




Describing the Location as part of the Establishing the Setting

   We can describe a location by listing and modifying
   its attributes. We can do this objectively with linking
   words like L 'has' or 'contains' or 'is full of'. We can
   do it subjectively from someone,e.g., the main character's POV. Here
   linking words would reflect p-conciousness like "loves" L's ... or
   "hates" L's .... We can choose this word based on the subjects psychological
   characteristics -- how he "feels" about a location 
	(likes, loves, hates, has enjoyed). Of course if a character hates a 
   location then the reader expects some explanation for why the character is there.

   Each object in the KB may have classes of linguistic modifiers associated with it.
   For example, clock-tower might have size modifers big, tall etc. shape and color
   modifiers that apply etc.
 
 

*/

%======================================================
% Write Sentence
%	Given the name of a sentence frame builds
%	a instance of that sentence returned as a list.
%======================================================

%---------------------------------------------
% Write a PARALLEL_SETTING sentence
%---------------------------------------------
write_sentence(parallel_setting,Sentence) :-
	lookup(setting,Sentence_Frame,Location),
	lookup(point_of_view,Sentence_Frame,POV),
	lookup(pc_verb,Sentence_Frame,Verb),
	lookup(tense,Sentence_Frame,Tense),
	lookup(code,Sentence_Frame,P),
	P(Location,POV,Verb,Tense,Sentence).
		
		

%======================================================
% Describe Setting.
%
%   Uses parallel structue to shape a 
%   sentence describing a setting location.
%   Randomly selects iconic features of
%   the location, modifying them and then
%   building a sentence connecting the list
%   to the location through a randonly selected
%   linking verb.
%  
%
%  Variations: Objective, POV(Character)
%
%=========================================

describe_setting(Loc,objective,Loc,S) :-
	nl,
	meta_splice(['The', Loc],NP),
	choose_verb(feature_link,Verb),
	build_modified_feature_list(Loc,MFL1),
	insert_and(MFL1,MFL2),
	meta_splice(MFL2,MFL),
	meta_splice([NP,Verb,MFL],S),
	writeseqnl(S).

%----------------------------------------------------------------------------------
% Builds a sentence about a setting highlighting
% a negative or threating feature of the setting. Used
% in contrast to previous mentioned positive features
% to warn the reader.
%
%   "Of course, the S has its *downside*. It has its
%    N1, N2 and N3. S's N2 can be as *merciless* as they
%    are in *worse place ever for N3s*."
%
%    NXs are features. Features should have iconic objects
%    -- things where that feature is highly suggestive. For
%    example, tests are highly suggestive in the context of
%    schools as opposed to playgrounds. This object of place
%    that features the negative aspect of the setting is called
%    the counter-setting of the setting with respect to the
%    feature.
%    For example, S = university, Negative-Feature(NF) = trials
%       counter-setting(S,NF) = marketplace.
%----------------------------------------------------------------------------------
forboding_contrast().


%----------------------------------------------------------------------------------
% Describe Setting based on characters POV.
% Must Choose P-Conciousness verb to establish
% characters conscious awareness of setting.
%
% Currently only present and past tense supported
%
%  TODO: Must decide to use positive or negative
%        literay modifiers based on "charge" of
%        PCVerb.
%
%----------------------------------------------------------------------------------
dspov(Location,Character,PCVerb,Tense,Sentences) :-
	tense(PCVerb,Tense,Verb),
	meta_splice([Character,Verb,`the`, Location],S1),
	get_pronoun(Character,sub,CPro),
	get_pronoun(Location,pos,LPro),
	build_possesive_modified_feature_list(Location,LPro,MFL),
	meta_splice([CPro,Verb | MFL],S2),
	append(S1,S2,Sentences),
	pp_sentence(S1),
	pp_sentence(S2).



%=================================================
% Sentence Type: Describe Persons Goal
%
%  Build a sentence that links a person to his/her
%  Goal.
%  Assumptions:
%	1) Goal is a expressed as a verb
%
%=================================================
describePersonsGoal(Person,Goal,Tense,[Person,LinkVerb,GoalVerb]) :-
	choose_verb(goal_link,Verb),
	tense(Verb, Tense, LinkVerb),
	tense(Goal,infinitive,GoalVerb).
	
			
%=========================================
%=========================================
build_possesive_modified_feature_list(Object,PPro,PMFL) :-
	build_modified_feature_list(Object,MFL),
	map_possesive(MFL,PPro,PMFL1),
	insert_and(PMFL1,PMFL2),
	meta_splice(PMFL2,PMFL).

%=========================================
%=========================================
build_modified_feature_list(Object,MFL) :-
	icons(positive,Object,Features),
	%lookupall(positive_icons,Object,Features),
	randomly_select_at_most(3,Features,NFeatures),
	modify_list(NFeatures,MFL).



%********************************************************
%
% 	Modified Feature List Operators
%
%********************************************************

%=========================================
% 
%=========================================
map_possesive(MFL,PPro,R) :-
	findall(V,( member(MF,MFL),V = [PPro | MF]),R).

%=========================================
% 
% Inserts an 'and' at beggining of
% last pair in list. Pairs are intended
% to represent [modifier object] pairs
% that will be listed in a sentence.
%
%=========================================
insert_and(LOL,R) :-
	reverse(LOL,RLOL),
	=(RLOL,[F | Rest]),
	append([and],F,NewLast),
	reverse([NewLast | Rest],R).


%====================================================
% MODIFY_LIST of Features
%
% Assign modifiers to each feature in a feature list
%
%====================================================
modify_list(Features,ModifiedFeatures) :-
	modify1(Features,[],ModifiedFeatures).

modify1([],ModifiedFeatures,ModifiedFeatures) :- !.
modify1([F | Features],ACCUM,ModifiedFeatures) :-
	modify(F,MF),
	modify1(Features, [MF | ACCUM], ModifiedFeatures).


%================================================================
% MODIFY Feature
%
% We can enhance this by constraining
% the selection of modifiers based on
% the object that has the feature.
% For example, university sports might
% be modified differently than professional sports.
%
% The Word Usage Pattern to use for this is the feature_modifier,
% rather than just the modifier,
% which is predicated on the Object as well as its feature.
%
%
% If the Feature can NOT be modified based on 
% knowledge base then default modifier is 'interesting'.
%
%================================================================
modify(Feature,[Modifier,Feature]) :-
	all_modifiers(positive,Feature,PLMs),
	not(=(PLMs,[])),!,
	randomly_select_at_most(1,PLMs,[Modifier]).

%--------------------------------------------------
% MODIFY Feature where feature was not recognized
%	and may need to be depluralized.
%
%--------------------------------------------------
modify(Feature,[Modifier,Feature]) :-
	depluralize(Feature,DePFeature),
	isa_frame(DePFeature,_),
	lookupall(positive_literary_modifiers,DePFeature,PLMs),!,
	randomly_select_at_most(1,PLMs,[Modifier]).

%--------------------------------------------------
% MODIFY Feature where feature is not a recognized
%	frame. Default modifier is 'interesing'.
%--------------------------------------------------
modify(Feature,[interesting,Feature]).
