%*****************************************
% 
%
%       Word Morphology Utilities
%
%
%*****************************************

%==================================
%Derivational Morphology Rules
%  For Verbs
%==================================

tense(Verb,present,Present) :-
	present(Verb,Present).

tense(Verb,past,PastVerb) :-
	past(Verb,PastVerb).

tense(Verb, infinitive, ToVerb) :-
	cat([to,' ',Verb],ToVerb,Joins).


%===========================================
% Present Tense 
%===========================================

%-----------------------
%Irregular
%-----------------------

%-----------------------
%Regular
%-----------------------
present(Verb,Present) :-
	name(Verb,Verb_Chars),
	name(s,Suffix_Chars),
	append(Verb_Chars,Suffix_Chars,Present_Chars),
	atom_chars(Present,Present_Chars).
	

%===========================================
% Past Tense 
%===========================================

%-----------------------
%Irregular
%-----------------------
past(go,went) :- !.
past(shoot,shot) :- !.
past(run,ran) :- !.
past(fall,fell) :- !.
past(hear,herd) :- !.
past(have,had) :- !.
past(win,won) :- !.
past(drive,drove) :- !.


%-----------------------
%ends in 'e'add 'd'
%-----------------------
past(Verb,Past) :-
	ends_in_vowel(Verb,e),!,
	name(Verb,Verb_Chars),
	name(d,Suffix_Chars),
	append(Verb_Chars,Suffix_Chars,Past_Chars),
	atom_chars(Past,Past_Chars).

%-----------------------
%regular
%add 'ed'
%-----------------------
past(Verb,Past) :-
	name(Verb,Verb_Chars),
	name(ed,Suffix_Chars),
	append(Verb_Chars,Suffix_Chars,Past_Chars),
	atom_chars(Past,Past_Chars).


%====================================
% Pronouns, Prepostions and Conjunctions
%     Derivational Morphology
% - Case, Gender, Number, Tense
%
%====================================

%----------------
%Initial Filter
% No Case or Gender Args
%----------------
pronoun(W) :-
	pronoun(Case,Gender,W).

%-------------
%List Filter
%-------------
pronoun(Case,Gender,List) :-
	list(List),!,
	List=[Word],
	nonvar(Word),
	pronoun(Case,Gender,Word).

%-------------------------
%- Case Specified Not a list.
%   Literal or Variable
%------------------------

pronoun(Case,W) :-
	pronoun(Case,_,W).


%
%Pronoun Facts
%
pronoun(Case,Gender,you).
pronoun(sub,Gender,i).
pronoun(obj,Gender,me).
pronoun(Case,Gender,it).
pronoun(_,_,someone).

%
%pronoun(Speaker,Case,Gender,Pronoun)
%
pronoun(1,sub,G,i) :- !.
	%pronoun(1,sub,f,i).
	%pronoun(1,sub,m,i).
	%pronoun(1,sub,n,i*).
pronoun(1,obj,G,me) :- !.
	%pronoun(1,obj,f,me).
	%pronoun(1,obj,m,me).
	%pronoun(1,obj,n,me*).
pronoun(1,pos,G,my) :- !.
	%pronoun(1,pos,f,my).
	%pronoun(1,pos,m,my).
	%pronoun(1,pos,n,my*).
%
% 2nd person, subjective
%
pronoun(2,sub,f,she).
pronoun(2,sub,m,he).
pronoun(2,sub,n,it).
%
% 2nd person, objective
%
pronoun(2,obj,f,her).
pronoun(2,obj,m,him).
pronoun(2,obj,n,it).
%
% 2nd person, posessive
%
pronoun(2,pos,f,her).
pronoun(2,pos,m,his).
pronoun(2,pos,n,its).


%---------------------
%Prepositions
%---------------------

%List Filter
%
prep(List) :-
	list(List),!,
	List=[Word],
	nonvar(Word),
	prep(Word).


/*
prep(Prep) :-
	prep(Class,Prep).
*/


prep(with).
prep(into).
prep(to).
prep(in).
prep(on).
prep(near).
prep(beneath).
prep(above).
prep(under).
prep(like).   %Is this a preposition?


%---------------------
%Conjunctions
%---------------------
conj(and).
conj(or).
conj(but).


%===================================
%Plural Rules
%===================================

%-----------------------
% Wx -> Wxes
%-----------------------
pluralize(Word,Plural) :-
	name(Word,Word_Chars),
	name(x,Ending),
	prefix_of(Ending,Word_Chars,Prefix),!,
	name(xes,Plural_Suffix),
	append(Prefix,Plural_Suffix,Plural_Chars),
	name(Plural,Plural_Chars).



%-----------------------
%WCy -> WCCies
%-----------------------
pluralize(Word,Plural) :-
	name(Word,Word_Chars),
	name(y,Last),
	append([C],Last,Ending),
	prefix_of(Ending,Word_Chars,Prefix),
	consonant_p(C),!,
	name(ies,S),
	append([C,C],S,Plural_Suffix),
	append(Prefix,Plural_Suffix,Plural_Chars),
	name(Plural,Plural_Chars).

%-----------------------
%Ws -> Ws^
%-----------------------
pluralize(Word,Plural) :-
	name(Word,Word_Chars),
	name(s,Last),
	Ending = Last,
	prefix_of(Ending,Word_Chars,Prefix),!,
	name('^',S),
	append(Last,S,Plural_Suffix),
	append(Prefix,Plural_Suffix,Plural_Chars),
	name(Plural,Plural_Chars).

%-----------------------
%W -> Ws
%-----------------------
pluralize(Word,Plural) :-
	name(Word,Word_Chars),
	name(s,Last),
	append(Word_Chars,Last,Plural_Chars),
	name(Plural,Plural_Chars).


depluralize(PWord,Word) :-
	name(PWord,PWord_Chars),
	name(xes,Ending),
	prefix_of(Ending,PWord_Chars,Base_Word_Chars),!,
	name(x,Return_Ending),
	append(Base_Word_Chars,Return_Ending,Word_Chars),
	name(Word,Word_Chars).

depluralize(PWord,Word) :-
	name(PWord,PWord_Chars),
	name(s,Last),
	prefix_of(Last,PWord_Chars,Word_Chars),!,
	name(Word,Word_Chars).


%*************************
%
%Vowel Utilities
%
%*************************
%
%
%
vowel_p(X) :-
	number(X),!,
	member(X,[97,101,105,111,117]).

vowel_p(X) :-
	atom(X),
	member(X,[a,e,i,o,u]),!.
	
%=========================	
%
%=========================	
ends_in_vowel(Word) :-
	last_char(Word,Last),
	vowel_p(Last).

ends_in_vowel(Word,Vowel) :-
	last_char(Word,Vowel),
	vowel_p(Vowel).

%*************************
%
%Consonant Utilities
%
%*************************
%
%
consonant_p(X) :-
	not(vowel_p(X)).


%********************************
%
%Prefix and Suffix Utilities
%
%********************************

prefix(W,P) :-
	append(P,_,W).

prefix_of(Suffix,Word,Prefix) :-
	append(Prefix,Suffix,Word). 

suffix(W,S) :-
	append(_,S,W).



%===============================
%
%===============================	
last_char(Word,Last) :-
	name(Word,Word_Chars),
	length(Word_Chars,Len),
	Len1 is Len - 1,
	cat(Parts,Word,[Len1]),
	=(Parts,[_,Last]).
	
append_period(Word,PWord) :-
	name(Word,Word_Chars),
	name(.,P),
	append(Word_Chars,P,PChars),
	name(PWord,PChars).

capitalize(Word,CWord) :-
	name(Word,[F | RChars]),
	UpF is F - 32,
	name(CWord,[UpF | RChars]).

	
upper_case(LC,UC) :-
	not(number(LC)),
	atom_chars(LC,[LN]),
	LN =< 122, 
	LN >= 97,
	UN is LN - 32,
	atom_chars(UC,[UN]). 	