/*

	          Word Usage Pattern (WUP) Relations

ToDo:

 Build higher-level predicates that use frame hierarchy to 
 implement inheritance of usage patterns over anchor words (i.e. objects).

 Finish encoding pattern instances based on current frame definitions.
 Change code to use these pattern relations
 Remove attributes from frames (now redundant with WUP Relations).

 Scan and automatically reverse engineer current stories to populate
 WUP relations. SO given a story it will learn WUP that it can
 reuse.

*/

%======================================================================
% PATTERN SCEHMA
%======================================================================
%% icons(positive, Object, Icons)
%
%% icons(negative, Object, Icons)
%
%% modifiers(positive, Object, Modifiers)
%
%% modifiers(negative, Object, Modifiers)
%
%% feature_modifiers(positive, Object, Iconic_Feature, Feature_Modifiers)
%
%% feature_modifiers(negative, Object, Iconic_Feature, Feature_Modifiers)
%
%% counter_setting(Original_Setting,Negative_Feature,Counter_Setting)


%======================================================================
% PATTERN Inheritance and Combination Rules
%======================================================================
all_icons(Charge,Anchor,Icons) :-
	findall(I,(ancestor(Anchor,A),icons(Charge,A,I)),NIcons),
	flatten(NIcons,Icons).

all_modifiers(Charge,Anchor,Modifiers) :-
	findall(I,(ancestor(Anchor,A),modifiers(Charge,A,I)),NMods),
	flatten(NMods,Modifiers).


%--------------------------------------------------------------------
% Returns feature modifiers predicated on object. If none exist
% then returns modifiers on feature independently of featured object.
% Inheritance rule is UNCLEAR since you can inherit along OBJECT
% or FEATURE Lines. MUST THINK MORE ON THIS.
%--------------------------------------------------------------------
all_feature_modifiers(Charge,Object,Feature,Modifiers) :-
	feature_modifiers(Charge, Object, Feature, Modifiers),!.

all_feature_modifiers(Charge,Object,Feature,Modifiers) :-
	modifiers(Charge, Feature, Modifiers),!.
	

all_feature_modifiers(Charge,Feature,Modifiers) :-
	modifiers(Charge, Feature, Modifiers),!.


%======================================================================
% PATTERN Instances
%======================================================================

%POSITIVE ICONS (ICONS of an oject are compnent features that are strongly
%			correlated with the object in a negative or positive
%			manner)
%
%
icons( positive, place, 	[ ambiance, vista, space]).
icons( positive, university, 	[ clocktowers, brick, ivy, youth, architecture,
				books, knowledge, sports, students]).
icons( positive, ivy, 		[ leaves, vines]).
icons( positive, book, 		[ knowledge, cover, pages]).
icons( positive, brick,		[ lines, symetry]).
icons( positive, youth, 	[ boys, girls]).
icons( positive, architecture, 	[ brick, beams, steeples, windows, cornerstones]).
icons( positive, clocktower, 	[ clock, hands, face, numbers, 
				second_hand, minute_hand, hour_hand]).

%NEGATIVE ICONS
%
icons( negative, architecture, 		[ cracks]).
icons( negative, university, 		[ tests, competition, 'intellectual snobery']).
icons( negative, book, 			[ pages]).
icons( negative, brick, 		[ points, edges]).
icons( negative, ivy, 			[ poison]).
icons( negative, place, 		[ feeling]).
icons( negative, 'business world',	[trials]).

%POSITIVE OBJECT MODIFIERS
%
modifiers( positive, architecture, 	[sturdy, ancient, grand, majestic]).
modifiers( positive, clocktower, 	[accurate, ivy-covered]).
modifiers( positive, ivy, 		[spreading, green, lush]).
modifiers( positive, brick, 		[solid, sturdy, elegant]).
modifiers( positive, book,  		[fascinating, mind_opening, wonderous]).
modifiers( positive, sports,		[fun_loving]).
modifiers( positive, knowledge,		[fascinating, deep]).
modifiers( positive, youth, 		[eager, studious, 'fun-loving']).


%NEGATIVE OBJECT MODIFIERS
%
modifiers( negative, book,  	[dirty, wordy, sensless]).
modifiers( negative, brick, 	[deadly, drab, dense]).
modifiers( negative, ivy, 	[poisonness, tangled]).
modifiers( negative, youth, 	[rambunctious, rebelious, naive]).
modifiers( negative, trials,	[stark, unforgiving, painful, laborious, burning]).
modifiers( negative, tests,	[merciless, biting, foul, `gut churning`]).
modifiers( negative, teachers,	[devious, sadistic, jaded]).


%POSITIVE FEATURE MODIFIERS
%
feature_modifiers( positive, brick, edges, [fine]).
feature_modifiers( positive, book, pages, [well_crafted]).
feature_modifiers( positive, university, sports, [amateur]).
feature_modifiers( positive, university, teachers, [dedicated, insightful, wise, inspiring]).
feature_modifiers( positive, university, books, ['resevoirs of', wonderous, brilliant]).


%NEGATIVE FEATURE MODIFIERS
%
feature_modifiers( negative, brick, edges, [cutting]).
feature_modifiers( negative, book, pages, [boring, dense]).
feature_modifiers( negative, place, feeling, [erie]).
feature_modifiers( negative, 'business world', trials, [stark, unforgiving, painful, 'cut-throat', 
				laborious]).


% Counter Settings
%
%
counter_setting(university, trials, 'business world'). 
counter_setting(university, deception, 'business world').


%synonym
%
%
synonym(X,Y) :-
	(syn(X,Y) ; syn(Y,X)),!.

synonym(X,X).

syn(trials, tests).
syn(trial, test).
syn(trials, 'coal walks').
syn(trials, challenges).
syn(university, academia).
syn(university, school).
syn('business world', marketplace).
